/**************************************************************************************************
* File: MP3_driver.c
* File Type: C - Source Code File
* Company: (c) mikroElektronika, 2011-2011
* Revision History:
*     20101121 (TL):
*       - initial release;
*     20110211 (FJ):
*       - fixed write/read to the shared SFR;
*     20110512:
*       - modified for HW revision 1.05 (FJ);
*     20111020:
*       - modified for mikroMedia PIC32 HW revision 1.10 (DA);
* Description:
*     This module contains a set of functions that are used for communication with
*     VS1053E mp3 codec.
* Test configuration:
*       MCU:             P32MX460F512L
*       Dev.Board:       Mikromedia_for_PIC32
*                        http://www.mikroe.com/eng/products/view/595/mikrommb-for-pic32-board/
*       Oscillator:      80000000 Hz (80MHz)
*       SW:              mikroC PRO for PIC32
*                        http://www.pic32compilers.com/
* NOTES:
*     Mp3 codec use SPI bus to communicate with MCU.
**************************************************************************************************/
#include <built_in.h>
#include "MP3_driver.h"
#include "MP3_Player_objects.h"
#include "MP3_Player_resources.h"
#include "built_in.h"

// Global Variables
unsigned long file_size;
unsigned long total_file_size;
const BYTES_2_WRITE = 32;
char mp3_buffer[BYTES_2_WRITE];
char volume_left, volume_right;
int volume = 50;
int volume_old = 50;
int FatType = 0;

// VS1053E constants
const char WRITE_CODE           = 0x02;
const char READ_CODE            = 0x03;

const char SCI_BASE_ADDR        = 0x00;
const char SCI_MODE_ADDR        = 0x00;
const char SCI_STATUS_ADDR      = 0x01;
const char SCI_BASS_ADDR        = 0x02;
const char SCI_CLOCKF_ADDR      = 0x03;
const char SCI_DECODE_TIME_ADDR = 0x04;
const char SCI_AUDATA_ADDR      = 0x05;
const char SCI_WRAM_ADDR        = 0x06;
const char SCI_WRAMADDR_ADDR    = 0x07;
const char SCI_HDAT0_ADDR       = 0x08;
const char SCI_HDAT1_ADDR       = 0x09;
const char SCI_AIADDR_ADDR      = 0x0A;
const char SCI_VOL_ADDR         = 0x0B;
const char SCI_AICTRL0_ADDR     = 0x0C;
const char SCI_AICTRL1_ADDR     = 0x0D;
const char SCI_AICTRL2_ADDR     = 0x0E;
const char SCI_AICTRL3_ADDR     = 0x0F;

/**************************************************************************************************
* MMC Chip Select connection
**************************************************************************************************/
sbit Mmc_Chip_Select at LATG9_bit;
sbit Mmc_Chip_Select_Direction at TRISG9_bit;

/**************************************************************************************************
* Serial Flash Chip Select connection
**************************************************************************************************/
sbit CS_Serial_Flash_bit at LATC2_bit;
sbit CS_Serial_Flash_Direction_bit at TRISC2_bit;

/**************************************************************************************************
* CODEC V1053E connections
**************************************************************************************************/
sbit MP3_CS_Direction          at TRISG15_bit;
sbit MP3_CS                    at LATG15_bit;
sbit MP3_RST_Direction         at TRISD9_bit;
sbit MP3_RST                   at LATD9_bit;
sbit DREQ_Direction            at TRISD8_bit;
sbit DREQ                      at RD8_bit;
sbit BSYNC_Direction           at TRISB8_bit;
sbit BSYNC                     at LATB8_bit;

int playing = 0;

int first = 1;

char nomFichier[25] = "";
short fileHandler =  0;


/**************************************************************************************************
* Function MP3_SCI_Write()
* -------------------------------------------------------------------------------------------------
* Overview: Function writes one byte to MP3 SCI
* Input: register address in codec, data
* Output: Nothing
**************************************************************************************************/
void MP3_SCI_Write(char address, unsigned int data_in) {
  BSYNC = 1;

  MP3_CS = 0;                    // select MP3 SCI
  SPI2_Write(WRITE_CODE);
  SPI2_Write(address);
  SPI2_Write(Hi(data_in));       // high byte
  SPI2_Write(Lo(data_in));       // low byte
  MP3_CS = 1;                    // deselect MP3 SCI
  while (DREQ == 0);             // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI
}

/**************************************************************************************************
* Function MP3_SCI_Read()
* -------------------------------------------------------------------------------------------------
* Overview: Function reads words_count words from MP3 SCI
* Input: start address, word count to be read
* Output: words are stored to data_buffer
**************************************************************************************************/
void MP3_SCI_Read(char start_address, char words_count, unsigned int *data_buffer) {
  unsigned int temp;

  MP3_CS = 0;                    // select MP3 SCI
  SPI2_Write(READ_CODE);
  SPI2_Write(start_address);

  while (words_count--) {        // read words_count words byte per byte
    temp = SPI2_Read(0);
    temp <<= 8;
    temp += SPI2_Read(0);
    *(data_buffer++) = temp;
  }
  MP3_CS = 1;                    // deselect MP3 SCI
  while (DREQ == 0);             // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI
}

/**************************************************************************************************
* Function MP3_SDI_Write()
* -------------------------------------------------------------------------------------------------
* Overview: Function write one byte to MP3 SDI
* Input: data to be writed
* Output: Nothing
**************************************************************************************************/
void MP3_SDI_Write(char data_) {

  MP3_CS = 1;
  BSYNC = 0;

  while (DREQ == 0);             // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI

  SPI2_Write(data_);
  BSYNC = 1;
}

/**************************************************************************************************
* Function MP3_SDI_Write_32
* -------------------------------------------------------------------------------------------------
* Overview: Function Write 32 bytes to MP3 SDI
* Input: data buffer
* Output: Nothing
**************************************************************************************************/
void MP3_SDI_Write_32(char *data_) {
  char i;

  MP3_CS = 1;
  BSYNC = 0;

  while (DREQ == 0);             // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI

  for (i=0; i<32; i++)
  SPI2_Write(data_[i]);
  BSYNC = 1;
}

/**************************************************************************************************
* Function MP3_Set_Volume()
* -------------------------------------------------------------------------------------------------
* Overview: Function set volume on the left and right channel
* Input: left channel volume, right channel volume
* Output: Nothing
**************************************************************************************************/
void MP3_Set_Volume(char left, char right) {
  unsigned int volume;

  volume = (left<<8) + right;             // calculate value
  MP3_SCI_Write(SCI_VOL_ADDR, volume);    // Write value to VOL register
}

void MP3_Init(void)
{
  BSYNC = 1;
  MP3_CS = 1;

  // Hardware reset
  MP3_RST = 0;
  Delay_ms(10);
  MP3_RST = 1;

  while (DREQ == 0);

  MP3_SCI_Write(SCI_MODE_ADDR, 0x0800);
  MP3_SCI_Write(SCI_BASS_ADDR, 0x7A00);
  MP3_SCI_Write(SCI_CLOCKF_ADDR, 0x2000);   // default 12 288 000 Hz

  // Volume depart: 30% ((1-(log10(30)/2))*255)
  volume_left  = 0x42; //0x3F;
  volume_right = 0x42; //0x3F;
  MP3_Set_Volume(volume_left, volume_right);
  //MP3_LoadSpectrumPlugin();
  MP3_CS = 0;
}

/**************************************************************************************************
* Function MP3_Start()
* -------------------------------------------------------------------------------------------------
* Overview: Function Initialize SPI to communicate with MP3 codec
* Input: Nothing
* Output: Nothing
**************************************************************************************************/
void MP3_Start(void)
{
  IC1CONbits.FEDGE = 1;           // Fronts montants
  IC1IP0_bit = 1;                 // interruptions de niveau 4
  IC1IP1_bit = 1;
  IC1IP2_bit = 1;
  IC1CONSETbits.ICI = 0b00;
  IC1CONSETbits.ICM = 0b001;
  IC1CONbits.ON = 1;
  IC1CONbits.SIDL = 0;
  IC1IE_bit = 1;                  // interruptions activees
  EnableInterrupts();
  // Disable other peripheral modules on the same SPI bus
  CS_Serial_Flash_Direction_bit = 0;
  CS_Serial_Flash_bit = 1;             // Disable Serial Flash module

  MP3_CS_Direction  = 0;               // Configure MP3_CS as output
  MP3_CS            = 1;               // Deselect MP3_CS
  MP3_RST_Direction = 0;               // Configure MP3_RST as output
  MP3_RST           = 1;               // Set MP3_RST pin

  DREQ_Direction    = 1;               // Configure DREQ as input
  BSYNC_Direction   = 0;               // Configure BSYNC as output
  BSYNC             = 0;               // Clear BSYNC
  BSYNC             = 1;               // Clear BSYNC

  SPI2_Init_Advanced(_SPI_MASTER, _SPI_8_BIT, 26, _SPI_SS_DISABLE, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_LOW, _SPI_IDLE_2_ACTIVE);

  MP3_Init();
  Delay_ms(1000);
}

int StartPlayingFile(char * fileName)
{
    int i = 0;
    int position = 0;
    int result = 0;
    if (FatType == 16) {
        result = Mmc_Fat_Assign(fileName, 0x01);
    }             /*
    else if(FatType == 32) {
        fileHandler = FAT32_Open(fileName, 0x01);
        if (fileHandler > 0) result = 1;
    }  */
    
    if (result == 1) {
      Mmc_Fat_Reset(&file_size);
      total_file_size = file_size;
      playing = 1;
      while (DREQ == 1) {
        for (i=0; i<BYTES_2_WRITE; i++)
        {
          if(FatType == 16) Mmc_Fat_Read(mp3_buffer+i);  /*
          else if (FatType == 32) FAT32_Read(fileHandler, mp3_buffer+i,1);   */
        }
        file_size-=BYTES_2_WRITE;
        MP3_SDI_Write_32(mp3_buffer);
      }
      position = (total_file_size - file_size) * 100 / total_file_size;
      if (ProgressBar1.position != position) {
        ProgressBar1.position = position;
        ProgressButton.Left = ProgressBar1.Left + ProgressBar1.position/100.0 * ProgressBar1.Width - ProgressButton.Width/2;
        DrawBox(&ProgressMasque);
        DrawProgressBar(&ProgressBar1);
        DrawRoundBox(&ProgressButton);
      }
      return 0;
    }
    else return -1;
}

void ContinuePlaying() {
      int i = 0;
      int position = 0;
      playing = 0;
      while(DREQ == 0);
      while ((DREQ == 1) && (file_size >= BYTES_2_WRITE)) {
        for (i=0; i<BYTES_2_WRITE; i++)
        {
          if(FatType == 16) Mmc_Fat_Read(mp3_buffer+i);    /*
          else if (FatType == 32) FAT32_Read(fileHandler, mp3_buffer+i,1);  */
        }
        file_size-=BYTES_2_WRITE;
        MP3_SDI_Write_32(mp3_buffer);
      }
      playing = 1;
      position = (total_file_size - file_size) * 100 / total_file_size;
      if (ProgressBar1.position != position) {
        ProgressBar1.position = position;
        ProgressButton.Left = ProgressBar1.Left + ProgressBar1.position/100.0 * ProgressBar1.Width - ProgressButton.Width/2;
        DrawBox(&ProgressMasque);
        DrawProgressBar(&ProgressBar1);
        DrawRoundBox(&ProgressButton);
      }
}

void ChangePosition(unsigned long position)
{
    if(FatType == 16) Mmc_Fat_Seek(position);       /*
    else if(FatType == 32) FAT32_Seek(fileHandler, position); */
}

void IC1_ISR() iv IVT_INPUT_CAPTURE_1 ilevel 7 ics ICS_SRS {
    long buff = 0;
    int i = 0;
    int position = 0;
    if (playing == 1) {
      while ((DREQ == 1) && (file_size >= BYTES_2_WRITE)) {
        for (i=0; i<BYTES_2_WRITE; i++)
        {
          if(FatType == 16) Mmc_Fat_Read(mp3_buffer+i);    /*
          else if (FatType == 32) FAT32_Read(fileHandler, mp3_buffer+i,1);   */
        }
        file_size-=BYTES_2_WRITE;
        MP3_SDI_Write_32(mp3_buffer);
      }
      if (file_size < BYTES_2_WRITE) {
        for (i=0; i<file_size; i++)
        {
          if(FatType == 16) Mmc_Fat_Read(mp3_buffer+i);          /*
          else if (FatType == 32) FAT32_Read(fileHandler, mp3_buffer+i,1);   */
        }
        for (i=0; i<file_size; i++)
        {
          MP3_SDI_Write(mp3_buffer+i);
        }
        playing = 0;
      }
      position = (total_file_size - file_size) * 100 / total_file_size;
      if (ProgressBar1.position != position) {
        ProgressBar1.position = position;
        ProgressButton.Left = ProgressBar1.Left + ProgressBar1.position/100.0 * ProgressBar1.Width - ProgressButton.Width/2;
        DrawBox(&ProgressMasque);
        DrawProgressBar(&ProgressBar1);
        DrawRoundBox(&ProgressButton);
      }
    }
    buff = IC1BUF;
    IC1IF_bit = 0;
}

/**************************************************************************************************
* End of File
**************************************************************************************************/