
#include "__Lib_FAT32.h"

////////////////////////////////////////////////////////////////////////////////
//
//  These functions are wrappers for hardware specific functions.
//
//  The intention is to write the library in such a way it can be
//  independent of the type of storage device.
//
//  User may modify these functions if required, but return values
//  and conditions should remain as stated in function descriptions.
//
//  Only SD/MMC cards are supported at this moment. More will come.
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//
//  Dev_Init() is a wrapper for initialization of the storage device.
//
//  It must return Returns OK on success and ERROR on failure.
//
////////////////////////////////////////////////////////////////////////////////
int8 FAT32_Dev_Init(void)
{
    if (OK != Mmc_Init())
    {
        return ERROR;
    }
    return OK;
}

////////////////////////////////////////////////////////////////////////////////
//
//  Dev_Read_Sector() a wrapper for read opeartions on the storage device.
//
//  It is used to read the contents of sector sc from the storage device
//  into the buffer buf.
//
//  It must return Returns OK on success and ERROR on failure.
//
////////////////////////////////////////////////////////////////////////////////
int8 FAT32_Dev_Read_Sector(__SECTOR sc, char* buf)
{
    if (Mmc_Read_Sector(sc, buf) != OK)
    {
        return ERROR;
    }
    return OK;
}

////////////////////////////////////////////////////////////////////////////////
//
//  Dev_Write_Sector() is a wrapper for read operations on the storage device.
//
//  It is used to write the contents of buffer buf into the sector sc on the
//  storage device.
//
//  It must return Returns OK on success and ERROR on failure.
//
////////////////////////////////////////////////////////////////////////////////
int8 FAT32_Dev_Write_Sector(__SECTOR sc, char* buf)
{
    if (Mmc_Write_Sector(sc, buf) != OK)
    {
        return ERROR;
    }
    return OK;
}

////////////////////////////////////////////////////////////////////////////////
//
//  Dev_Size() is a wrapper for retrieving number of sector device implements.
//
//  It is used to get sector count of the device (for fat formatting purpose,
//  assumed sector size is 512 bytes).
//
//  Enter your code for retrieving device size.
//
//  It must return Returns OK on success and ERROR on failure.
//
////////////////////////////////////////////////////////////////////////////////
int8 FAT32_Dev_SectorCount(uint32 *scCnt)
{
    uint8  csdbuf[16];
    uint16 c_size, c_size_mult, mult,
           read_bl_len, block_len;
    uint32 size, blocknr;

    // determine MMC/SD card size in MB
    if (Mmc_Read_Csd(csdbuf) != OK)
    {
        return ERROR;
    }
    
    // is it version 2.0?
    if (1 == ((csdbuf[0] & 0xC0) >> 6))
    {
        size  = 0;                  size <<= 8;
        size += csdbuf[7] & 0x3F;   size <<= 8;
        size += csdbuf[8];          size <<= 8;
        size += csdbuf[9];          size <<= 0;

        // size is in 0.5MB, get size in sectors (assumed 512 bytes sector size)
        size *= 1024;
    }
    // if not, it's version 1.xx
    else
    {
        c_size      = ((csdbuf[8] & 0xC0) >> 6) +
                      ((unsigned) csdbuf[7] << 2) +
                      (((unsigned) csdbuf[6] & 0x03) << 10);
        c_size_mult = (csdbuf[10] & 0x80) + 
                      (((unsigned) csdbuf[9] & 0x03) << 8);
        c_size_mult = c_size_mult >> 7;
        
        read_bl_len = csdbuf[5] & 0x0f;
        
        mult = 1;
        mult = mult << (c_size_mult + 2);
        
        blocknr = (c_size + 1) * (long) mult;
        block_len = 1;
        block_len = block_len << read_bl_len;
        
        size = block_len * blocknr;

        // size is in 1B, get size in sectors (assumed 512 bytes sector size)
        size /= 512;
    }
    
    *scCnt = size;

    return OK;
}

////////////////////////////////////////////////////////////////////////////////
//
//  Multi sector read is MMC/SD card feature which enables you to read
//  any number of consecutive sectors starting from specified sector.
//  Multi sector read algorithm:
//    1. Issue start command with start sector number.
//       Device will enter Multi sector read mode.
//    2. Read n sectors...
//    3. Issue stop command when done reading.
//       Device will exit Multi sector read and enter idle mode.
//
//  If your device does not support or you don't want to use this feature 
//  comment MULTI_SECTOR_READ define below.
//
////////////////////////////////////////////////////////////////////////////////
#define MULTI_SECTOR_READ
#ifdef MULTI_SECTOR_READ
////////////////////////////////////////////////////////////////////////////////
//
//  Dev_Multi_Read_Star() a wrapper for multi read start operation (if any) on
//  the storage device.
//
//  It is used to intiate multi read operation starting from sector sc 
//  on the storage device.
//
//  It must return Returns OK on success and ERROR on failure.
//
////////////////////////////////////////////////////////////////////////////////
int8 FAT32_Dev_Multi_Read_Start(__SECTOR sc)
{
    if (Mmc_Multi_Read_Start(sc) != OK)
    {
        return ERROR;
    }
    return OK;
}

////////////////////////////////////////////////////////////////////////////////
//
//  Dev_Multi_Read_Stop() a wrapper for multi read stop operation (if any) on
//  the storage device.
//
//  It is used to stop multi read operation on the storage device.
//
//  It must return Returns OK on success and ERROR on failure.
//
////////////////////////////////////////////////////////////////////////////////
int8 FAT32_Dev_Multi_Read_Stop()
{
    if (Mmc_Multi_Read_Stop() != OK)
    {
        return ERROR;
    }
    return OK;
}

////////////////////////////////////////////////////////////////////////////////
//
//  Dev_Multi_Read_Sector() a wrapper for sector read operation in
//  'multi sector read' storage device (if any).
//
//  It is used to read the contents of next sector from the storage device
//  into the specified buffer.
//
//  It must return Returns OK on success and ERROR on failure.
//
////////////////////////////////////////////////////////////////////////////////
int8 FAT32_Dev_Multi_Read_Sector(char* buf)
{
    Mmc_Multi_Read_Sector(buf);
    return OK;
}

#else
////////////////////////////////////////////////////////////////////////////////
// override multi sector read routines for devices which do not have this feature
////////////////////////////////////////////////////////////////////////////////
static __SECTOR __sc = 0;
////////////////////////////////////////////////////////////////////////////////
//
//  Dev_Multi_Read_Star() a wrapper for multi read start operation (if any) on
//  the storage device.
//
//  It is used to intiate multi read operation starting from sector sc
//  on the storage device.
//
//  It must return Returns OK on success and ERROR on failure.
//
////////////////////////////////////////////////////////////////////////////////
int8 FAT32_Dev_Multi_Read_Start(__SECTOR sc)
{
    __sc = sc;
    return OK;
}

////////////////////////////////////////////////////////////////////////////////
//
//  Dev_Multi_Read_Stop() a wrapper for multi read stop operation (if any) on
//  the storage device.
//
//  It is used to stop multi read operation on the storage device.
//
//  It must return Returns OK on success and ERROR on failure.
//
////////////////////////////////////////////////////////////////////////////////
int8 FAT32_Dev_Multi_Read_Stop()
{
    return OK;
}

////////////////////////////////////////////////////////////////////////////////
//
//  Dev_Multi_Read_Sector() a wrapper for sector read operation in
//  'multi sector read' storage device (if any).
//
//  It is used to read the contents of next sector from the storage device
//  into the buffer buf.
//
//  It must return Returns OK on success and ERROR on failure.
//
////////////////////////////////////////////////////////////////////////////////
int8 FAT32_Dev_Multi_Read_Sector(char* buf)
{
    return FAT32_Dev_Read_Sector(__sc++, buf);
}
#endif

////////////////////////////////////////////////////////////////////////////////
//
//  Dev_PutChar() is used by FAT32_Dir function for printing the contents
//  of the current folder, which is done 1 character at the time.
//
//  User may change this function to reroute that on some other device,
//  GLCD, TFT, Flash, EEPROM, or Ethernet module, to name a few.
//
//  It must return Returns OK on success and ERROR on failure.
//
////////////////////////////////////////////////////////////////////////////////
int8 FAT32_Put_Char(char ch)
{
    UART_Write(ch);
    return OK;
}
////////////////////////////////////////////////////////////////////////////////
//
//  Dev_GetChar() is currently not required by any FAT32 function.
//
//  It is just considered useful to have it for future extensions.
//
//  It must return Returns OK on success and ERROR on failure.
//
////////////////////////////////////////////////////////////////////////////////
int8 FAT32_Get_Char(char *ch)
{
    if (1 == UART_Data_Ready())
    {
        *ch = UART_Read();
        return OK;
    }
    return ERROR;
}
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////